const DutchAuction = artifacts.require("DutchAuction");

contract("DutchAuction", (accounts) => {
    let dutchAuction;
    const names = ["Item 1", "Item 2"];
    const startPrices = ["2000000000000000000", "1000000000000000000"]; // 2 ether, 1 ether
    const endPrices = ["300000000000000000", "150000000000000000"];  // 0.3 ether, 0.15 ether
    const durations = ["20400", "20400"]; // 340 minutes, 340 minutes
    const intervals = ["1200", "1200"]; // 20 minutes, 20 minutes

    const currentTimestamp = Math.floor(Date.now() / 1000);

    before(async () => {

        dutchAuction = await DutchAuction.new(
            names,
            currentTimestamp,
            startPrices,
            endPrices,
            durations,
            intervals
        );

        //dutchAuction = await DutchAuction.deployed();
    });

    it("should deploy the contract", async () => {
        assert.ok(dutchAuction.address);
    });


    it("should get current price", async () => {
        const currentPrice = await dutchAuction.getCurrentPrice.call();
        console.log("Current Price:", currentPrice.toString());
        assert.ok(currentPrice.toString() > 0, "Current price should be non-negative");
    });

    it("should return the correct start timestamp", async () => {
        const startTimestamp = await dutchAuction.getStartTimestamp.call();
        assert.equal(startTimestamp, currentTimestamp, "Start timestamp should match the provided value");
    });

    it("should return the correct price after 20 minutes", async () => {
        const tenMinutesInSeconds = 1200;

        await increaseTime(tenMinutesInSeconds);

        const currentPriceAfter = await dutchAuction.getCurrentPrice.call();

        console.log("Price after 10 minutes ", currentPriceAfter.toString());
        assert.equal(currentPriceAfter.toString(), "1900000000000000000", "Price after 20 minutes should be correct");
    });

    it("should return the correct price after 330 minutes", async () => {
        const tenMinutesInSeconds = 19800 - 1200;

        await increaseTime(tenMinutesInSeconds);

        const currentPriceAfter = await dutchAuction.getCurrentPrice.call();

        console.log("Price after 330 minutes ", currentPriceAfter.toString());
        assert.equal(currentPriceAfter.toString(), "400000000000000000", "Price after 330 minutes should be correct");
    });

    it("should return the correct price after 340 minutes", async () => {
        const tenMinutesInSeconds = 600;

        await increaseTime(tenMinutesInSeconds);

        const currentPriceAfter = await dutchAuction.getCurrentPrice.call();

        console.log("Price after 340 minutes ", currentPriceAfter.toString());
        assert.equal(currentPriceAfter.toString(), "1000000000000000000", "Price after 340 minutes should be correct");
    });

    it("should return the correct price after 670 minutes", async () => {
        const tenMinutesInSeconds = 19800;

        await increaseTime(tenMinutesInSeconds);

        const currentPriceAfter = await dutchAuction.getCurrentPrice.call();

        console.log("Price after 670 minutes ", currentPriceAfter.toString());
        assert.equal(currentPriceAfter.toString(), "200000000000000000", "Price after 670 minutes should be correct");
    });

    it("should allow buying the item", async () => {
        const currentPrice = await dutchAuction.getCurrentPrice.call();

        const buyer = accounts[0];
        const buyingPrice = currentPrice;

        const buyTransaction = await dutchAuction.buy({from: buyer, value: buyingPrice});

        assert.ok(buyTransaction.receipt.status, "Transaction should be successful");

        const currentIndex = await dutchAuction.currentItemIndex.call();

        const newItem = await dutchAuction.items(currentIndex);
        const newItemBuyer = newItem.buyer;

        assert.equal(newItemBuyer, buyer, "Buyer should be updated");
    });

    it("should recover the funds", async () => {
        const buyer = accounts[0];
        const recoveryTransaction = await dutchAuction.recoverFunds({from: buyer});

        assert.ok(recoveryTransaction.receipt.status, "Transaction should be successful");
    });

});

function increaseTime(duration) {
    const id = Date.now();
    return new Promise((resolve, reject) => {
        web3.currentProvider.send({
            jsonrpc: "2.0",
            method: "evm_increaseTime",
            params: [duration],
            id: id,
        }, err1 => {
            if (err1) return reject(err1);
            web3.currentProvider.send({
                jsonrpc: "2.0",
                method: "evm_mine",
                id: id + 1,
            }, (err2, res) => {
                return err2 ? reject(err2) : resolve(res);
            });
        });
    });
}