"use client"

import React, { useEffect, useState } from 'react';
import { useRouter } from 'next/navigation';
import { Typography, Card, CardContent, Button, Box } from '@mui/material';

import NavBar from '../components/NavBar';
import { getAuctionContract, checkConnectionMetaMask } from '../utils';

const CloseAuctionPage = () => {
    const { auctionContract } = getAuctionContract();
    const router = useRouter();
    const [auctionStatus, setAuctionStatus] = useState('');
    const [isAllClosed, setIsAllClosed] = useState(false);
    const [isRecovered, setIsRecovered] = useState(false);

    useEffect(() => {
        const checkStatus = async () => {
            const accounts = await checkConnectionMetaMask();
            if (accounts.length > 0) {
                try {
                    const allAuctionsClosed = await auctionContract.methods.isEnded().call();
                    setAuctionStatus(allAuctionsClosed ? 'Toutes les enchères sont fermées' : 'En attente de fermeture');
                    setIsAllClosed(allAuctionsClosed);
                    const allFundsRecovered = await auctionContract.methods.isRecovered().call();
                    setIsRecovered(allFundsRecovered);
                    if (allFundsRecovered) {
                        setAuctionStatus('Tous les fonds ont été récupérés');
                    }
                } catch (error) {
                    console.error('Error checking auction status:', error);
                }
            }
        };
        checkStatus();
    }, []);

    const handleAuctionClosure = async () => {
        try {
            await auctionContract.methods.recoverFunds().send({ from: window.ethereum.selectedAddress });
            setIsRecovered(true);
            setAuctionStatus('Tous les fonds ont été récupérés');
            console.log('Auction closed successfully.');
        } catch (error) {
            console.error('Error closing auction:', error);
        }
    };

    return (
        <>
            <NavBar />
            <Box m={2}>
                <Card>
                    <CardContent>
                        <Typography variant="h5">Versement des fonds de toutes les enchères</Typography>
                        <Typography variant="body1">
                            Cette page vous permet d'effectuer le versement des fonds de toutes les enchères.
                        </Typography>
                        <Typography variant="subtitle1">Statut global : {auctionStatus}</Typography>
                        <Button
                            variant="contained"
                            color="primary"
                            disabled={!isAllClosed || isRecovered}
                            onClick={handleAuctionClosure}
                        >
                            Versement des fonds de toutes les enchères
                        </Button>
                    </CardContent>
                </Card>
            </Box>
        </>
    );
};

export default CloseAuctionPage;
