"use client";

import React, { useState, useEffect } from 'react';
import { Typography, TextField, Button, Container } from '@mui/material';
import Web3 from 'web3';
import { useRouter } from 'next/navigation';

import NavBar from '../components/NavBar';
import { checkConnectionMetaMask, getAuctionContract } from '../utils';

const CreateAuction = () => {
    const { auctionContract, web3 } = getAuctionContract();
    const [auctionDetails, setAuctionDetails] = useState({
        title: '',
        startPrice: '',
        duration: '',
        reservePrice: '',
        interval: '',
    });
    const router = useRouter();

    const handleInputChange = (e) => {
        const { name, value } = e.target;
        setAuctionDetails({
            ...auctionDetails,
            [name]: value,
        });
    };

    const handleCreateAuction = async () => {
        try {
            await auctionContract.methods.createAuction(
                auctionDetails.title,
                Web3.utils.toWei(auctionDetails.startPrice, 'ether'),
                Web3.utils.toWei(auctionDetails.reservePrice, 'ether'),
                auctionDetails.duration * 60,
                auctionDetails.interval * 60,
            ).send({ from: window.ethereum.selectedAddress, gas: 3000000 });

            window.location.href = '/auction-list';
        } catch (error) {
            console.error('Erreur lors de la création de l\'enchère:', error);
        }
    };

    useEffect(() => {
        const checkConnection = async () => {
            const accounts = await checkConnectionMetaMask();
            if (accounts.length === 0) {
                router.push('/');
            }
        };
        checkConnection();
    }, []);

    return (
        <>
            <NavBar />
            <Container maxWidth="sm">
                <Typography variant="h4" align="center" gutterBottom>
                    Créer une enchère
                </Typography>
                <TextField
                    label="Titre"
                    fullWidth
                    margin="normal"
                    name="title"
                    value={auctionDetails.title}
                    onChange={handleInputChange}
                />
                <TextField
                    label="Prix de départ (en Ether)"
                    fullWidth
                    margin="normal"
                    name="startPrice"
                    value={auctionDetails.startPrice}
                    onChange={handleInputChange}
                />
                <TextField
                    label="Durée (en minutes)"
                    fullWidth
                    margin="normal"
                    name="duration"
                    value={auctionDetails.duration}
                    onChange={handleInputChange}
                />
                <TextField
                    label="Prix de réserve (en Ether)"
                    fullWidth
                    margin="normal"
                    name="reservePrice"
                    value={auctionDetails.reservePrice}
                    onChange={handleInputChange}
                />
                <TextField
                    label="Intervalle de modification (en minutes)"
                    fullWidth
                    margin="normal"
                    name="interval"
                    value={auctionDetails.interval}
                    onChange={handleInputChange}
                />
                <Button variant="contained" color="primary" onClick={handleCreateAuction}>
                    Créer l'enchère
                </Button>
            </Container>
        </>
    );
};

export default CreateAuction;
