"use client";

import React, {useEffect, useState} from 'react';
import {useSDK} from '@metamask/sdk-react';
import {Typography, Box, Grid} from '@mui/material';
import Web3 from 'web3';
import { useRouter } from 'next/navigation';

import NavBar from '../components/NavBar';
import { checkConnectionMetaMask,  getAuctionContract } from '../utils';

const WalletInfo = () => {
    const {sdk, connected} = useSDK();
    const [account, setAccount] = useState(null);
    const [balance, setBalance] = useState(null);
    const router = useRouter();

    useEffect(() => {
        const fetchWalletInfo = async () => {
            const accounts = await checkConnectionMetaMask();
            console.log("accounts", accounts.length);
            if (accounts.length === 0) {
                router.push('/');
            } else {
                try {
                    const web3 = new Web3(window.ethereum);
                    const accounts = await web3.eth.getAccounts();
                    console.log("accounts", accounts[0]);
                    setAccount(accounts?.[0]);

                    const balanceWei = await web3.eth.getBalance(accounts[0]);
                    console.log("Balance in Wei:", balanceWei);

                    const balanceEth = web3.utils.fromWei(balanceWei, 'ether');
                    const formattedBalance = parseFloat(balanceEth).toFixed(4);
                    console.log("Formatted Balance in ETH:", formattedBalance);

                    setBalance(formattedBalance);

                } catch (error) {
                    console.error('Error fetching wallet information:', error);
                }
            }
        };
        fetchWalletInfo();
    }, []);

    return (
        <>
            <NavBar />
            <Grid container justifyContent="center" alignItems="center" style={{minHeight: '100vh'}}>
                <Grid item xs={12} textAlign="center">
                    <Typography variant="h4" color="primary">Info sur le portefeuille</Typography>
                    <Typography variant="h6">Adresse du portefeuille :</Typography>
                    <Typography variant="body1">{account}</Typography>
                    <Typography variant="h6">Solde en ETH :</Typography>
                    <Typography variant="body1">{balance} ETH</Typography>
                </Grid>
            </Grid>
        </>
    );
};

export default WalletInfo;
